﻿using GarmentFactoryContracts.BindingModels;
using GarmentFactoryContracts.ViewModels;
using GarmentFactoryDataModels.Enums;
using GarmentFactoryDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Threading.Tasks;

namespace GarmentFactoryDatabaseImplement.Models
{
	[DataContract]
	public class Order : IOrderModel
    {
		[DataMember]
		public int Id { get; set; }

		[DataMember]
		[Required]
        public int ClientId { get; private set; }

		[DataMember]
		[Required]
        public int TextileId { get; private set; }

		[DataMember]
		public int? ImplementerId { get; private set; }

		[DataMember]
		[Required]
        public int Count { get; set; }

		[DataMember]
		[Required]
        public double Sum { get; private set; }

		[DataMember]
		[Required]
        public OrderStatus Status { get; private set; } = OrderStatus.Неизвестен;

		[DataMember]
		[Required]
        public DateTime DateCreate { get; private set; } = DateTime.Now;

		[DataMember]
		public DateTime? DateComplete { get; private set; }

        // Для передачи названия изделия
        public virtual Textile Textile { get; set; }

        // Для передачи ФИО клиента
        public virtual Client Client { get; set; }

		public virtual Implementer? Implementer { get; set; } = new();

		public static Order? Create(OrderBindingModel model)
        {
            if (model == null)
            {
                return null;
            }
			using var context = new GarmentFactoryDatabase();
			return new Order()
            {
                Id = model.Id,
                ClientId = model.ClientId,
                TextileId = model.TextileId,
				ImplementerId = model.ImplementerId,
				Implementer = model.ImplementerId.HasValue ? context.Implementers.First(x => x.Id == model.ImplementerId) : null,
				Count = model.Count,
                Sum = model.Sum,
                Status = model.Status,
                DateCreate = model.DateCreate,
                DateComplete = model.DateComplete,
			};
        }

        public void Update(OrderBindingModel model)
        {
            if (model == null)
            {
                return;
            }
			using var context = new GarmentFactoryDatabase();
			ImplementerId = model.ImplementerId;
            Implementer = model.ImplementerId.HasValue ? context.Implementers.First(x => x.Id == model.ImplementerId) : null;
			Status = model.Status;
            DateComplete = model.DateComplete;
            context.SaveChanges();
        }

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            ClientId = ClientId,
            TextileId = TextileId,
			ImplementerId = ImplementerId,
			Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateComplete = DateComplete,
            TextileName = Textile.TextileName,
			ClientFIO = Client.ClientFIO,
            ClientEmail = Client.Email,
			ImplementerFIO = Implementer?.ImplementerFIO,
		};
    }
}
